@@ -27,6 +27,7 @@
 #include <libhal.h>
 #include <libhal-storage.h>
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
 #include <libnautilus-extension/nautilus-extension-types.h>
@@ -42,153 +43,6 @@ static GType gmp_type = 0;
 static void property_page_provider_iface_init (NautilusPropertyPageProviderIface *iface);
 static GList *gmp_properties_get_pages (NautilusPropertyPageProvider *provider, GList *files);
 
-/* Implementation of the NautilusInfoProvider interface */
-
-/* nautilus_info_provider_update_file_info 
- * This function is called by Nautilus when it wants the extension to 
- * fill in data about the file.  It passes a NautilusFileInfo object,
- * which the extension can use to read data from the file, and which
- * the extension should add data to.
- *
- * If the data can be added immediately (without doing blocking IO), 
- * the extension can do so, and return NAUTILUS_OPERATION_COMPLETE.  
- * In this case the 'update_complete' and 'handle' parameters can be 
- * ignored.
- * 
- * If waiting for the deata would block the UI, the extension should
- * perform the task asynchronously, and return 
- * NAUTILUS_OPERATION_IN_PROGRESS.  The function must also set the 
- * 'handle' pointer to a value unique to the object, and invoke the
- * 'update_complete' closure when the update is done.
- * 
- * If the extension encounters an error, it should return 
- * NAUTILUS_OPERATION_FAILED.
- */
-static NautilusOperationResult
-update_file_info (NautilusInfoProvider *provider,
-		  NautilusFileInfo *file,
-		  GClosure *update_complete,
-		  NautilusOperationHandle **handle)
-{
-	char *volume_hal_udi = NULL;
-	char *drive_hal_udi = NULL;
-	GnomeVFSDrive *drive = NULL;
-	GnomeVFSVolume *volume = NULL;
-	NautilusOperationResult result;
-
-
-	result = NAUTILUS_OPERATION_COMPLETE;
-
-
-	drive = nautilus_file_info_get_drive (file);
-	if (drive != NULL)
-		gnome_vfs_drive_ref (drive);
-
-	volume = nautilus_file_info_get_volume (file);
-	if (volume != NULL) {
-		gnome_vfs_volume_ref (volume);
-	} else {
-		if (drive != NULL)
-			volume = gnome_vfs_drive_get_mounted_volume (drive);
-	}
-
-	if (drive == NULL && volume != NULL)
-		drive = gnome_vfs_volume_get_drive (volume);
-
-	if (drive != NULL)
-		drive_hal_udi = gnome_vfs_drive_get_hal_udi (drive);
-	if (volume != NULL)
-		volume_hal_udi = gnome_vfs_volume_get_hal_udi (volume);
-
-	if (volume != NULL && volume_hal_udi != NULL) {
-		char *backing_udi;
-		DBusError error;
-
-		dbus_error_init (&error);
-		backing_udi = libhal_device_get_property_string (
-			hal_ctx,
-			volume_hal_udi,
-			"volume.crypto_luks.clear.backing_volume",
-			&error);
-		if (backing_udi != NULL) {
-			nautilus_file_info_add_emblem (file, "lock");
-			g_message ("setting lock for %x", file);
-			libhal_free_string (backing_udi);
-		}
-
-	}
-
-	if (drive_hal_udi != NULL)
-		g_free (drive_hal_udi);
-	if (volume_hal_udi != NULL)
-		g_free (volume_hal_udi);
-	if (drive != NULL)
-		gnome_vfs_drive_unref (drive);
-	if (volume != NULL)
-		gnome_vfs_volume_unref (volume);
-
-	return result;
-
-#if 0
-	NautilusCvsDirectory *directory;
-	NautilusOperationResult result;
-	
-	directory = get_cvs_directory (file);
-
-	/* Keep the directory alive for as long as the file is around */
-	watch_directory (directory, file);
-
-	if (nautilus_cvs_directory_is_ready (directory)) {
-		update_file_info (file, directory);
-		
-		result = NAUTILUS_OPERATION_COMPLETE;
-	} else {
-		NautilusCvsHandle *cvs_handle;
-		
-		cvs_handle = g_new0 (NautilusCvsHandle, 1);
-
-		cvs_handle->provider = provider;
-		cvs_handle->file = g_object_ref (file);
-
-		/* We need to keep the closure around until we call it
-		 * back. */
-		cvs_handle->update_complete = g_closure_ref (update_complete);
-
-		nautilus_cvs_directory_call_when_ready 
-			(directory, directory_ready_callback, cvs_handle);
-
-		*handle = (NautilusOperationHandle*)cvs_handle;
-
-		result = NAUTILUS_OPERATION_IN_PROGRESS;
-	}
-
-	g_object_unref (directory);
-
-	return result;
-#endif
-}
-
-static void
-cancel_update (NautilusInfoProvider *provider,
-	       NautilusOperationHandle *handle)
-{
-#if 0
-	NautilusCvsHandle *cvs_handle;
-	
-	cvs_handle = (NautilusCvsHandle*)handle;
-	
-	cvs_handle->cancelled = TRUE;
-#endif
-}
-
-static void 
-info_provider_iface_init (NautilusInfoProviderIface *iface)
-{
-	iface->update_file_info = update_file_info;
-	iface->cancel_update = cancel_update;
-}
-
-
 static void
 gnome_mount_properties_plugin_register_type (GTypeModule *module)
 {
@@ -211,12 +65,6 @@ gnome_mount_properties_plugin_register_type (GTypeModule *module)
 		NULL
 	};
 
-	static const GInterfaceInfo info_provider_iface_info = {
-		(GInterfaceInitFunc) info_provider_iface_init,
-		NULL,
-		NULL
-	};
-
 	gmp_type = g_type_module_register_type (module, G_TYPE_OBJECT,
 			"GnomeMountPropertiesPlugin",
 			&info, 0);
@@ -225,11 +73,6 @@ gnome_mount_properties_plugin_register_type (GTypeModule *module)
 				     gmp_type,
 				     NAUTILUS_TYPE_PROPERTY_PAGE_PROVIDER,
 				     &property_page_provider_iface_info);
-
-	g_type_module_add_interface (module,
-				     gmp_type,
-				     NAUTILUS_TYPE_INFO_PROVIDER,
-				     &info_provider_iface_info);
 }
 
 static void
@@ -248,55 +91,82 @@ gmp_properties_get_pages (NautilusPropertyPageProvider *provider,
 	GList *pages = NULL;
 	char *uri = NULL;
 	NautilusFileInfo *file;
-	GnomeVFSDrive *drive = NULL;
-	GnomeVFSVolume *volume = NULL;
 	GtkWidget *page;
 	GtkWidget *label;
 	NautilusPropertyPage *property_page;
 	char *volume_hal_udi = NULL;
 	char *drive_hal_udi = NULL;
 	gboolean added_volume;
+        GFile *location;
 
 	/* only add properties page if a single file is selected */
 	if (files == NULL || files->next != NULL)
 		goto out;
 	file = files->data;
 
-	/*g_message ("name = '%s'", nautilus_file_info_get_name (file));*/
+        /* TODO: There is currently no way to get the GVolume or
+         * GDrive for resp. unmounted volumes and drives without media
+         * (in computer:///) for a NautilusFileInfo object.
+         *
+         * http://bugzilla.gnome.org/show_bug.cgi?id=518700
+         */
 
-	drive = nautilus_file_info_get_drive (file);
-	if (drive != NULL)
-		gnome_vfs_drive_ref (drive);
+        location = nautilus_file_info_get_location (file);
+        drive_hal_udi = NULL; volume_hal_udi = NULL;
 
-	volume = nautilus_file_info_get_volume (file);
-	if (volume != NULL) {
-		gnome_vfs_volume_ref (volume);
-	} else {
-		if (drive != NULL)
-			volume = gnome_vfs_drive_get_mounted_volume (drive);
-	}
+        if (nautilus_file_info_get_file_type (file) == G_FILE_TYPE_SHORTCUT ||
+            nautilus_file_info_get_file_type (file) == G_FILE_TYPE_MOUNTABLE) {
+                char *activation_uri;
+                activation_uri = nautilus_file_info_get_activation_uri (file);
+                if (activation_uri != NULL) {
+                        GFile *activation_file;
+                        GVolumeMonitor *volume_monitor;
 
-	if (drive == NULL && volume != NULL)
-		drive = gnome_vfs_volume_get_drive (volume);
+                        activation_file = g_file_new_for_uri (activation_uri);
+                        volume_monitor = g_volume_monitor_get ();
+                        if (volume_monitor != NULL) {
+                                GList *mounts, *l;
+                                mounts = g_volume_monitor_get_mounts (volume_monitor);
+                                for (l = mounts; l != NULL; l = l->next) {
+                                        GMount *mount = l->data;
+                                        GVolume *volume;
+                                        GFile *mount_root;
 
-	if (drive != NULL)
-		drive_hal_udi = gnome_vfs_drive_get_hal_udi (drive);
-	if (volume != NULL)
-		volume_hal_udi = gnome_vfs_volume_get_hal_udi (volume);
+                                        mount_root = g_mount_get_root (mount);
+                                        if (g_file_equal (mount_root, activation_file)) {
+                                                volume = g_mount_get_volume (mount);
+                                                if (volume != NULL) {
+                                                        volume_hal_udi = g_volume_get_identifier (volume, 
+                                                                                                  G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
+                                                        g_object_unref (volume);
+                                                }
+                                                g_object_unref (mount);
+                                        }
+                                        g_object_unref (mount_root);
+                                }
+                                if (mounts != NULL)
+                                        g_list_free (mounts);
+                        }
+                        g_object_unref (volume_monitor);
+                        g_object_unref (activation_file);
+                        
+                }
+                g_free (activation_uri);
+        }
+        g_object_unref (location);
 
 	added_volume = FALSE;
-	if (volume != NULL && volume_hal_udi != NULL) {
+	if (volume_hal_udi != NULL) {
 		char *udi;
 		LibHalVolume *vol;
-		udi = gnome_vfs_volume_get_hal_udi (volume);
-		vol = libhal_volume_from_udi (hal_ctx, udi);
-		g_free (udi);
+		vol = libhal_volume_from_udi (hal_ctx, volume_hal_udi);
 		if (vol != NULL) {
-			g_debug ("volume=%p vol=%p, vol->udi=%s", 
-				 volume, vol, vol != NULL ? libhal_volume_get_udi (vol) : "none");
+                        if (drive_hal_udi == NULL)
+                                drive_hal_udi = g_strdup (libhal_volume_get_storage_device_udi (vol));
+
 			page = gm_properties_view_new ();
 			gm_properties_view_set_info_volume (GM_PROPERTIES_VIEW (page), vol, hal_ctx);
-			label = gtk_label_new ("Volume");
+			label = gtk_label_new (_("Volume"));
 			gtk_widget_show (page);
 			property_page = nautilus_property_page_new ("gnome-mount-volume-properties", label, page);
 			pages = g_list_prepend (pages, property_page);
@@ -305,16 +175,15 @@ gmp_properties_get_pages (NautilusPropertyPageProvider *provider,
 		}
 	}
 
-	if (drive != NULL && drive_hal_udi) {
+	if (drive_hal_udi != NULL) {
 		char *udi;
-		char *udi2;
+		const char *udi2;
 		LibHalDrive *drv;
 
-		udi = gnome_vfs_drive_get_hal_udi (drive);
-		drv = libhal_drive_from_udi (hal_ctx, udi);
-		if (drv == NULL) {
+		drv = libhal_drive_from_udi (hal_ctx, drive_hal_udi);
+		if (drv == NULL && volume_hal_udi != NULL) {
 			LibHalVolume *vol;
-			vol = libhal_volume_from_udi (hal_ctx, udi);
+			vol = libhal_volume_from_udi (hal_ctx, volume_hal_udi);
 			if (vol != NULL) {
 				udi2 = libhal_volume_get_storage_device_udi (vol);
 				drv = libhal_drive_from_udi (hal_ctx, udi2);
@@ -323,11 +192,9 @@ gmp_properties_get_pages (NautilusPropertyPageProvider *provider,
 				 * show volume dialog anyway then!
 				 */
 				if (!added_volume) {
-					g_debug ("volume=%p vol=%p, vol->udi=%s", 
-						 volume, vol, vol != NULL ? libhal_volume_get_udi (vol) : "none");
 					page = gm_properties_view_new ();
 					gm_properties_view_set_info_volume (GM_PROPERTIES_VIEW (page), vol, hal_ctx);
-					label = gtk_label_new ("Volume");
+					label = gtk_label_new (_("Volume"));
 					gtk_widget_show (page);
 					property_page = nautilus_property_page_new ("gnome-mount-volume-properties", label, page);
 					pages = g_list_prepend (pages, property_page);
@@ -336,15 +203,11 @@ gmp_properties_get_pages (NautilusPropertyPageProvider *provider,
 				libhal_volume_free (vol);
 			}
 		}
-		g_free (udi);
 
 		if (drv != NULL) {
-			g_debug ("drive=%p  drv=%p, drv->udi=%s", 
-				 drive,  drv, drv != NULL ? libhal_drive_get_udi (drv) : "none");
-
 			page = gm_properties_view_new ();
 			gm_properties_view_set_info_drive (GM_PROPERTIES_VIEW (page), drv, hal_ctx);
-			label = gtk_label_new ("Drive");
+			label = gtk_label_new (_("Drive"));
 			gtk_widget_show (page);
 			property_page = nautilus_property_page_new ("gnome-mount-drive-properties", label, page);
 			pages = g_list_prepend (pages, property_page);
@@ -355,14 +218,8 @@ gmp_properties_get_pages (NautilusPropertyPageProvider *provider,
 
 out:
 
-	if (drive_hal_udi != NULL)
-		g_free (drive_hal_udi);
-	if (volume_hal_udi != NULL)
-		g_free (volume_hal_udi);
-	if (drive != NULL)
-		gnome_vfs_drive_unref (drive);
-	if (volume != NULL)
-		gnome_vfs_volume_unref (volume);
+        g_free (drive_hal_udi);
+        g_free (volume_hal_udi);
 
 	return pages;
 }
@@ -421,7 +278,7 @@ error:
 void
 nautilus_module_initialize (GTypeModule  *module)
 {
-	g_print ("Initializing gnome-mount extension\n");
+	//g_print ("Initializing gnome-mount extension\n");
 
 	hal_ctx = do_hal_init ();
 	if (hal_ctx == NULL) {
@@ -444,7 +301,7 @@ error:
 void
 nautilus_module_shutdown (void)
 {
-	g_print ("Shutting down gnome-mount extension\n");
+	//g_print ("Shutting down gnome-mount extension\n");
 }
 
 /* List all the extension types.  */
