@@ -29,6 +29,12 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <string.h>
+#include <stdlib.h>
+
+#include <dbus/dbus.h>
+#include <dbus/dbus-glib-lowlevel.h>
+#include <libhal.h>
+#include <libhal-storage.h>
 
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
@@ -45,12 +51,6 @@
 #define NOTIFY_EXPIRES_DEFAULT -1
 #endif
 
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
-#include <libhal.h>
-#include <libhal-storage.h>
-
 #if !defined(sun) && !defined(__FreeBSD__)
 #include <mntent.h>
 #elif defined(__FreeBSD__)
@@ -76,6 +76,7 @@ static int rc;
 static gboolean opt_noui = FALSE;
 static gboolean opt_block = FALSE;
 static gboolean opt_nodisplay = FALSE;
+static gboolean opt_verbose = FALSE;
 
 #define DBUS_TIMEOUT G_MAXINT
 
@@ -185,7 +186,7 @@ show_error_dialog_no_media (const char *udi, LibHalVolume *volume, LibHalDrive *
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (w),
 							  _("There is probably no media in the drive."));
 		gtk_dialog_run (GTK_DIALOG (w));
-		gtk_widget_unref (w);
+		gtk_widget_destroy (w);
 	}
 }
 
@@ -204,7 +205,7 @@ show_error_dialog_eject (const char *udi, LibHalVolume *volume, LibHalDrive *dri
 		 *   org.freedesktop.Hal.Device.Volume.InvalidEjectOption
 		 *   org.freedesktop.Hal.Device.Volume.Busy
 		 *   org.freedesktop.Hal.Device.Volume.UnknownFailure
-		 *
+		 *   org.freedesktop.Hal.Device.UnknownError
 		 *
 		 * in a sane way.
 		 */
@@ -239,13 +240,16 @@ show_error_dialog_eject (const char *udi, LibHalVolume *volume, LibHalDrive *dri
 			/* TODO: could add 'Lazy Unmount' button */
 		} else if (strcmp (error_name, "org.freedesktop.Hal.Device.Volume.InvalidUnmountOption") == 0 ||
 			   strcmp (error_name, "org.freedesktop.Hal.Device.Volume.UnsupportedEjectOption") == 0 ||
-			   strcmp (error_name, "org.freedesktop.Hal.Device.Volume.UnknownFailure") == 0) {
-			gtk_message_dialog_format_secondary_text (
+			   strcmp (error_name, "org.freedesktop.Hal.Device.Volume.UnknownFailure") == 0 ||
+			   strcmp (error_name, "org.freedesktop.Hal.Device.UnknownError") == 0) {
+			gtk_message_dialog_format_secondary_markup (
 				GTK_MESSAGE_DIALOG (w),
-				volume_name != NULL ? 
-				_("Cannot eject the volume '%s'.") :
-				_("Cannot eject the volume."),
-				volume_name);
+				_("There was an error ejecting the volume or drive."
+                                  "\n"
+                                  "\n"
+                                  "<b>%s:</b> %s"),
+                                error_name,
+                                error_detail);
 		}
 
 		gtk_dialog_add_buttons (GTK_DIALOG (w),
@@ -254,7 +258,7 @@ show_error_dialog_eject (const char *udi, LibHalVolume *volume, LibHalDrive *dri
 					NULL);
 
 		gtk_dialog_run (GTK_DIALOG (w));
-		gtk_widget_unref (w);
+		gtk_widget_destroy (w);
 	}
 }
 
@@ -370,7 +374,7 @@ show_error_dialog_unmount (const char *udi, LibHalVolume *volume, LibHalDrive *d
 					NULL);
 
 		gtk_dialog_run (GTK_DIALOG (w));
-		gtk_widget_unref (w);
+		gtk_widget_destroy (w);
 	}
 }
 
@@ -460,7 +464,7 @@ show_error_dialog_mount (LibHalVolume *volume, LibHalDrive *drive,
 		} else if (strcmp (error_name, "org.freedesktop.Hal.Device.Volume.AlreadyMounted") == 0) {
 
 			/* don't show any dialog for this */
-			gtk_widget_unref (w);
+			gtk_widget_destroy (w);
 			w = NULL;
 
 #if 0
@@ -481,7 +485,7 @@ show_error_dialog_mount (LibHalVolume *volume, LibHalDrive *drive,
 			 * and the volume contains signatures are not
 			 * yet cleaned out...
 			 */
-			gtk_widget_unref (w);
+			gtk_widget_destroy (w);
 			w = NULL;
 			 
 		} else if (strcmp (error_name, "org.freedesktop.DBus.Error.UnknownMethod") == 0) {
@@ -490,7 +494,7 @@ show_error_dialog_mount (LibHalVolume *volume, LibHalDrive *drive,
 			 *
 			 * Don't show any dialog for this.
 			 */
-			gtk_widget_unref (w);
+			gtk_widget_destroy (w);
 			w = NULL;
 			 
 		} else {
@@ -526,7 +530,7 @@ show_error_dialog_mount (LibHalVolume *volume, LibHalDrive *drive,
 						NULL);
 			
 			response = gtk_dialog_run (GTK_DIALOG (w));
-			gtk_widget_unref (w);
+			gtk_widget_destroy (w);
 			
 			switch (response) {
 				/* case MOUNT_ERROR_DIALOG_RESPONSE_INSTALL_DRIVER:
@@ -769,7 +773,8 @@ volume_mount_with_options (const char *udi, LibHalVolume *volume, LibHalDrive *d
 	dbus_error_init (&error);
 	if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, DBUS_TIMEOUT, &error)) ||
 	    dbus_error_is_set (&error)) {
-		g_warning ("Mount failed for %s\n%s : %s\n", udi, error.name, error.message);
+                if (opt_verbose)
+                        g_message ("Mount failed for %s\n%s : %s\n", udi, error.name, error.message);
 
 		if (strcmp (error.name, "org.freedesktop.Hal.Device.Volume.InvalidMountpoint") == 0 &&
 		    mount_point != NULL) {
@@ -778,7 +783,8 @@ volume_mount_with_options (const char *udi, LibHalVolume *volume, LibHalDrive *d
 			if (reply != NULL)
 				dbus_message_unref (reply);
 
-			g_warning ("Given mount point name '%s' was invalid, trying without this...", mount_point);
+                        if (opt_verbose)
+                                g_message ("Given mount point name '%s' was invalid, trying without this...", mount_point);
 
 			/* possibly illegal mount point name; try without that mount point name... */
 			ret = volume_mount_with_options (udi, volume, drive, NULL, fstype, options);
@@ -799,8 +805,9 @@ volume_mount_with_options (const char *udi, LibHalVolume *volume, LibHalDrive *d
 
 			new_mount_point = g_strdup_printf ("%s_", mount_point);
 
-			g_warning ("Given mount point name '%s' is unavailable, trying with '%s'...", 
-				   mount_point, new_mount_point);
+                        if (opt_verbose)
+                                g_message ("Given mount point name '%s' is unavailable, trying with '%s'...", 
+                                           mount_point, new_mount_point);
 
 			/* possibly illegal mount point name; try without that mount point name... */
 			ret = volume_mount_with_options (udi, volume, drive, new_mount_point, fstype, options);
@@ -834,7 +841,8 @@ volume_mount_with_options (const char *udi, LibHalVolume *volume, LibHalDrive *d
 
 	if ((device_file = get_dev_file (volume, drive)) != NULL) {
 		if ((mounted_at = get_mntent_mount_point(device_file)) != NULL) {
-			g_print (_("Mounted %s at \"%s\"\n"), device_file, mounted_at);
+                        if (opt_verbose)
+                                g_print (_("Mounted %s at \"%s\"\n"), device_file, mounted_at);
 			g_free (mounted_at);
 		}
 	}
@@ -1158,8 +1166,9 @@ volume_mount (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 			char *args[3] = {MOUNT, NULL, NULL};
 			char **envp = {NULL};
 
-			g_print (_("Device %s is in /etc/fstab with mount point \"%s\"\n"), 
-				 device_file, mount_point_fstab);
+                        if (opt_verbose)
+                                g_print (_("Device %s is in /etc/fstab with mount point \"%s\"\n"), 
+                                         device_file, mount_point_fstab);
 			args[1] = mount_point_fstab;
 			if (!g_spawn_sync ("/",
 					   args,
@@ -1179,8 +1188,9 @@ volume_mount (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 			if (exit_status != 0) {
 				char errstr[] = "mount: unknown filesystem type";
 
-				g_warning ("%s said error %d, stdout='%s', stderr='%s'\n", 
-					   MOUNT, exit_status, sout, serr);
+                                if (opt_verbose)
+                                        g_message ("%s said error %d, stdout='%s', stderr='%s'\n", 
+                                                   MOUNT, exit_status, sout, serr);
 
 				notify_parent (FALSE);
 
@@ -1201,7 +1211,8 @@ volume_mount (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 
 			}
 
-			g_print (_("Mounted %s at \"%s\" (using /etc/fstab)\n"), device_file, mount_point_fstab);
+                        if (opt_verbose)
+                                g_print (_("Mounted %s at \"%s\" (using /etc/fstab)\n"), device_file, mount_point_fstab);
 
 			g_free (mount_point_fstab);
 			ret = TRUE;
@@ -1718,8 +1729,9 @@ volume_unmount (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 			char *args[3] = {UMOUNT, NULL, NULL};
 			char **envp = {NULL};
 
-			g_print (_("Device %s is in /etc/fstab with mount point \"%s\"\n"),
-				 device_file, mount_point);
+                        if (opt_verbose)
+                                g_print (_("Device %s is in /etc/fstab with mount point \"%s\"\n"),
+                                         device_file, mount_point);
 			args[1] = mount_point;
 			if (!g_spawn_sync ("/",
 					   args,
@@ -1737,8 +1749,9 @@ volume_unmount (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 			}
 
 			if (exit_status != 0) {
-				g_warning ("%s said error %d, stdout='%s', stderr='%s'\n", 
-					   UMOUNT, exit_status, sout, serr);
+                                if (opt_verbose)
+                                        g_message ("%s said error %d, stdout='%s', stderr='%s'\n", 
+                                                   UMOUNT, exit_status, sout, serr);
 				g_free (mount_point);
 
 				if (strstr (serr, "is busy") != NULL) {
@@ -1754,7 +1767,8 @@ volume_unmount (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 				goto out;
 			}
 
-			g_print (_("Unmounted %s (using /etc/fstab)\n"), device_file);
+                        if (opt_verbose)
+                                g_print (_("Unmounted %s (using /etc/fstab)\n"), device_file);
 
 			g_free (mount_point);
 			ret = TRUE;
@@ -1818,7 +1832,8 @@ volume_unmount (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 		if (!is_ro) {
 			unmount_cache_timeout_cancel (FALSE);
 		}
-		g_warning ("Unmount failed for %s: %s : %s\n", udi, error.name, error.message);
+                if (opt_verbose)
+                        g_message ("Unmount failed for %s: %s : %s\n", udi, error.name, error.message);
 
 		if (strcmp (error.name, "org.freedesktop.Hal.Device.PermissionDeniedByPolicy") == 0) {
                         /* attempt to gain privilege */
@@ -1834,7 +1849,8 @@ volume_unmount (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 		goto out;
 	}
 
-	g_print (_("Unmounted %s\n"), get_dev_file (volume, drive));
+        if (opt_verbose)
+                g_print (_("Unmounted %s\n"), get_dev_file (volume, drive));
 	if (!is_ro) {
 		unmount_cache_timeout_cancel (TRUE);
 	}
@@ -1933,8 +1949,9 @@ volume_eject (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 			char *args[3] = {"eject", NULL, NULL};
 			char **envp = {NULL};
 
-			g_print (_("Device %s is in /etc/fstab with mount point \"%s\"\n"), 
-				 device_file, mount_point);
+                        if (opt_verbose)
+                                g_print (_("Device %s is in /etc/fstab with mount point \"%s\"\n"), 
+                                         device_file, mount_point);
 			args[1] = mount_point;
 			if (!g_spawn_sync ("/",
 					   args,
@@ -1952,8 +1969,9 @@ volume_eject (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 			}
 
 			if (exit_status != 0) {
-				g_warning ("%s said error %d, stdout='%s', stderr='%s'\n", 
-					   "eject", exit_status, sout, serr);
+                                if (opt_verbose)
+                                        g_message ("%s said error %d, stdout='%s', stderr='%s'\n", 
+                                                   "eject", exit_status, sout, serr);
 				g_free (mount_point);
 
 				if (strstr (serr, "is busy") != NULL) {
@@ -1969,7 +1987,8 @@ volume_eject (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 				goto out;
 			}
 
-			g_print (_("Ejected %s (using /etc/fstab).\n"), device_file);
+                        if (opt_verbose)
+                                g_print (_("Ejected %s (using /etc/fstab).\n"), device_file);
 
 			g_free (mount_point);
 			ret = TRUE;
@@ -2032,7 +2051,8 @@ volume_eject (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 		if (!is_ro) {
 			unmount_cache_timeout_cancel (FALSE);
 		}
-		g_warning ("Eject failed for %s: %s : %s\n", udi, error.name, error.message);
+                if (opt_verbose)
+                        g_message ("Eject failed for %s: %s : %s\n", udi, error.name, error.message);
 
 		if (strcmp (error.name, "org.freedesktop.Hal.Device.PermissionDeniedByPolicy") == 0) {
                         /* attempt to gain privilege */
@@ -2048,7 +2068,8 @@ volume_eject (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 		goto out;
 	}
 
-	g_print (_("Ejected %s\n"), get_dev_file (volume, drive));
+        if (opt_verbose)
+                g_print (_("Ejected %s\n"), get_dev_file (volume, drive));
 	if (!is_ro) {
 		unmount_cache_timeout_cancel (TRUE);
 	}
@@ -2141,7 +2162,7 @@ get_password (const char *udi, LibHalVolume *volume, LibHalDrive *drive, gboolea
 	LibHalDrive     *drv;
 	char            *model;
 	char            *vendor;
-
+        char            *title;
 
 	keyring_password = NULL;
 	keyring_is_session = FALSE;
@@ -2198,7 +2219,8 @@ get_password (const char *udi, LibHalVolume *volume, LibHalDrive *drive, gboolea
 	}
 
 	if (opt_noui) {
-		g_warning ("Not showing password dialog since invoked with --no-ui");
+                if (opt_verbose)
+                        g_message ("Not showing password dialog since invoked with --no-ui");
 		goto out;
 	}
 
@@ -2210,11 +2232,23 @@ get_password (const char *udi, LibHalVolume *volume, LibHalDrive *drive, gboolea
 	 */
 	notify_parent (TRUE);
 
-	prompt = g_strdup_printf (
-		_("The storage device %s contains encrypted data. Enter a password to unlock."), drive_name);
-	
-	dialog = gnome_password_dialog_new (_("Unlock Encrypted Data"), prompt, NULL, NULL, FALSE);
+        if (libhal_volume_is_partition (volume)) {
+                prompt = g_strdup_printf (_("The storage device %s contains encrypted data on partition %d. "
+                                            "Enter a password to unlock."),
+                                          drive_name,
+                                          libhal_volume_get_partition_number (volume));
+                title = g_strdup_printf (_("Unlock Encrypted Data (partition %d)"),
+                                         libhal_volume_get_partition_number (volume));
+        } else {
+                prompt = g_strdup_printf (_("The storage device %s contains encrypted data. "
+                                            "Enter a password to unlock."),
+                                          drive_name);
+                title = g_strdup_printf (_("Unlock Encrypted Data"));
+        }
+
+	dialog = gnome_password_dialog_new (title, prompt, NULL, NULL, FALSE);
 	g_free (prompt);
+        g_free (title);
 
 	gnome_password_dialog_set_show_username (GNOME_PASSWORD_DIALOG (dialog), FALSE);
 	gnome_password_dialog_set_show_userpass_buttons (GNOME_PASSWORD_DIALOG (dialog), FALSE);
@@ -2239,7 +2273,7 @@ get_password (const char *udi, LibHalVolume *volume, LibHalDrive *drive, gboolea
 		gnome_password_dialog_set_password (GNOME_PASSWORD_DIALOG (dialog), keyring_password);
 	}
 
-	gtk_widget_show (dialog);
+	gtk_window_present (GTK_WINDOW (dialog));
 
 	if (gnome_password_dialog_run_and_block (GNOME_PASSWORD_DIALOG (dialog)))
 	{
@@ -2301,7 +2335,8 @@ setup_crypto (const char *udi, LibHalVolume *volume, LibHalDrive *drive,
 	dbus_error_init (&error);
 	if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, msg, DBUS_TIMEOUT, &error)) || 
 	    dbus_error_is_set (&error)) {
-		g_warning ("Setup failed for %s: %s : %s\n", udi, error.name, error.message);
+                if (opt_verbose)
+                        g_message ("Setup failed for %s: %s : %s\n", udi, error.name, error.message);
 		if (strcmp (error.name, "org.freedesktop.Hal.Device.Volume.Crypto.SetupPasswordError") == 0) {
 			*password_error = TRUE;
 		}
@@ -2312,7 +2347,8 @@ setup_crypto (const char *udi, LibHalVolume *volume, LibHalDrive *drive,
 
 	ret = TRUE;
 
-	g_print (_("Setup clear-text device for %s.\n"), get_dev_file (volume, drive));
+        if (opt_verbose)
+                g_print (_("Setup clear-text device for %s.\n"), get_dev_file (volume, drive));
 
 out:
 	if (msg != NULL)
@@ -2378,7 +2414,8 @@ teardown_crypto (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 		goto out;
 	}
 
-	g_print (_("Teared down clear-text device for %s.\n"), get_dev_file (volume, drive));
+        if (opt_verbose)
+                g_print (_("Teared down clear-text device for %s.\n"), get_dev_file (volume, drive));
 
 	ret = TRUE;
 
@@ -2424,7 +2461,8 @@ crypto_setup_device_added (LibHalContext *ctx, const char *udi)
 
 				g_debug ("%s is backed by %s - will mount", udi, backing_udi);
 
-				g_print (_("Clear text device is %s. Mounting.\n"), get_dev_file (volume, NULL));
+                                if (opt_verbose)
+                                        g_print (_("Clear text device is %s. Mounting.\n"), get_dev_file (volume, NULL));
 					
 				fsuage = libhal_volume_get_fsusage (volume);
 					
@@ -2432,7 +2470,8 @@ crypto_setup_device_added (LibHalContext *ctx, const char *udi)
 					if (volume_mount (udi, volume, NULL))
 						crypto_setup_rc = 0;
 				} else {
-					g_warning ("%s does not have a mountable filesystem", udi);
+                                        if (opt_verbose)
+                                                g_message ("%s does not have a mountable filesystem", udi);
 				}
 				
 				notify_parent (crypto_setup_rc == 0);
@@ -2448,7 +2487,8 @@ crypto_setup_device_added (LibHalContext *ctx, const char *udi)
 static gboolean
 crypto_setup_timeout (gpointer data)
 {
-	g_warning ("Timeout for waiting for cleartext device... Exiting.");
+        if (opt_verbose)
+                g_message ("Timeout for waiting for cleartext device... Exiting.");
 	notify_parent (FALSE);
 	gtk_main_quit ();
 	
@@ -2691,7 +2731,6 @@ main (int argc, char *argv[])
 	char *resolved_device_file = NULL;
 	static gboolean opt_connect_crypto = FALSE;
 	static gboolean opt_disconnect_crypto = FALSE;
-	static gboolean opt_verbose = FALSE;
 	static gboolean opt_unmount = FALSE;
 	static gboolean opt_eject = FALSE;
 	static gchar *opt_hal_udi = NULL;
@@ -2757,10 +2796,12 @@ main (int argc, char *argv[])
 		opt_nodisplay = TRUE;
 	}
 
-	g_print ("%s\n", PACKAGE_STRING);
+        if (opt_verbose)
+                g_print ("%s\n", PACKAGE_STRING);
 
 	if (!gtk_init_check (&argc, &argv)) {
-		g_print (_("X display not available - using text-based operation.\n"));
+                if (opt_verbose)
+                        g_print (_("X display not available - using text-based operation.\n"));
 		opt_nodisplay = TRUE;
 	}
 
@@ -2857,7 +2898,8 @@ main (int argc, char *argv[])
 		if (volume != NULL) {
 			g_print (_("Resolved pseudonym \"%s\" -> %s\n"), opt_nickname, get_dev_file (volume, NULL));
 		} else {
-			g_warning (_("Cannot resolve pseudonym \"%s\" to a volume\n"), opt_nickname);
+                        if (opt_verbose)
+                                g_message (_("Cannot resolve pseudonym \"%s\" to a volume\n"), opt_nickname);
 			goto out;
 		}
 	} else {
@@ -2909,8 +2951,46 @@ main (int argc, char *argv[])
 			if (found) {
 				goto try_drive;
 			} else {
-				g_warning (_("Drive %s does not contain media."), 
-					 resolved_device_file != NULL ? resolved_device_file : opt_hal_udi);
+                                if (opt_eject) {
+                                        char **volumes_udis;
+                                        int num_volumes;
+                                        int n;
+
+                                        /* HACK TODO XXX ALERT: ugh.. try to find a mountable volume with an eject
+                                         * method..
+                                         */
+                                        volumes_udis = libhal_drive_find_all_volumes (hal_ctx, drive, &num_volumes);
+                                        if (volumes_udis != NULL) {
+                                                for (n = 0; n < num_volumes; n++) {
+                                                        char **vol_ifs;
+                                                        int m;
+
+                                                        vol_ifs = libhal_device_get_property_strlist (
+                                                                hal_ctx,
+                                                                volumes_udis[n],
+                                                                "info.interfaces",
+                                                                NULL);
+                                                        if (vol_ifs != NULL) {
+                                                                for (m = 0; vol_ifs[m] != NULL; m++) {
+                                                                        if (strcmp (ifs[m], "org.freedesktop.Hal.Device.Volume") == 0) {
+                                                                                volume = libhal_volume_from_udi (hal_ctx, volumes_udis[n]);
+                                                                                libhal_drive_free (drive);
+                                                                                drive = NULL;
+                                                                                goto try_drive;
+                                                                        }
+                                                                }
+                                                        }
+                                                        libhal_free_string_array (vol_ifs);
+                                                }
+                                                libhal_free_string_array (volumes_udis);
+                                        }
+
+                                }
+
+
+                                if (opt_verbose)
+                                        g_message (_("Drive %s does not contain media."), 
+                                                   resolved_device_file != NULL ? resolved_device_file : opt_hal_udi);
 				
 				notify_parent (FALSE);
 				
@@ -2919,8 +2999,9 @@ main (int argc, char *argv[])
 
 		} else {
 			/* Silently fail */
-			g_warning (_("Given device '%s' is not a volume or a drive."), 
-				 resolved_device_file != NULL ? resolved_device_file : opt_hal_udi);
+                        if (opt_verbose)
+                                g_message (_("Given device '%s' is not a volume or a drive."), 
+                                           resolved_device_file != NULL ? resolved_device_file : opt_hal_udi);
 		}
 
 		goto out;
@@ -2954,7 +3035,8 @@ try_drive:
 	}
 
 	if (opt_unmount && opt_eject) {
-		g_warning (_("Cannot unmount and eject simultaneously"));
+                if (opt_verbose)
+                        g_message (_("Cannot unmount and eject simultaneously"));
 		goto out;
 	}
 
@@ -2996,8 +3078,8 @@ try_drive:
 		}
 
 	} else if (opt_eject) {
-		if (volume_eject (udi, volume, drive))
-			rc = 0;
+                if (volume_eject (udi, volume, drive))
+                        rc = 0;
 	} else {
 		LibHalVolumeUsage fsusage;
 
@@ -3018,17 +3100,31 @@ try_drive:
 			/* see if we're already setup */
 			clear_udi = libhal_volume_crypto_get_clear_volume_udi (hal_ctx, volume);
 			if (clear_udi != NULL) {
-				g_warning (_("Crypto volume '%s' is already setup with clear volume '%s'"), 
-					 udi, clear_udi);
-				
+                                if (opt_verbose)
+                                        g_message (_("Crypto volume '%s' is already setup with clear volume '%s'"), 
+                                                   udi, clear_udi);
+
 				free (clear_udi);
 				goto out;
 			}
-			
+
+                        /* don't prompt for password if the device is locked
+                         * (e.g. https://bugzilla.redhat.com/show_bug.cgi?id=437309 )
+                         */
+                        if (libhal_device_is_locked_by_others (hal_ctx,
+                                                               libhal_volume_get_udi (volume),
+                                                               "org.freedesktop.Hal.Device.Storage",
+                                                               NULL)) {
+                                if (opt_verbose)
+                                        g_message (_("Crypto device %s is locked"),
+                                                   libhal_volume_get_udi (volume));
+                                goto out;
+                        }
+
 			g_debug ("Crypto volume - UDI '%s'", udi);
-			
+
 			setup_success = FALSE;
-			
+
 			/* we need this to catch when the cleartext device is added */
 			crypto_setup_backing_udi = udi;
 			libhal_ctx_set_device_added (hal_ctx, crypto_setup_device_added);
@@ -3043,13 +3139,15 @@ try_drive:
 			if (password != NULL) {
 				setup_success = setup_crypto (udi, volume, drive, password, &password_error);
 				if (!setup_success && password_error) {
-					g_warning (_("Bad crypto password"));
+                                        if (opt_verbose)
+                                                g_message (_("Bad crypto password"));
 					if (!opt_nodisplay && password_num_tries < 3) {
 						g_free (password);
 						password_retry = TRUE;
 						goto retry_password;
 					} else {
-						g_warning (_("Bailing out..."));
+                                                if (opt_verbose)
+                                                        g_message (_("Bailing out..."));
 					}
 				}
 			}
